@@ -1,4 +1,4 @@
-package Pcore::PDF v0.5.0;
+package Pcore::PDF v0.5.1;
 
 use Pcore -dist, -class, -const, -res;
 use Config;
@@ -126,9 +126,12 @@ sub generate_pdf ( $self, $html, $cb = undef ) {
 
     push $self->{_queue}->@*, [ $html, $on_finish ];
 
-    $self->_run_thread if $self->{_threads} < $self->{max_threads};
-
-    $self->{_signal}->send;
+    if ( $self->{_signal}->awaited ) {
+        $self->{_signal}->send;
+    }
+    elsif ( $self->{_threads} < $self->{max_threads} ) {
+        $self->_run_thread;
+    }
 
     return $rouse_cb ? Coro::rouse_wait $rouse_cb : ();
 }
@@ -165,8 +168,6 @@ sub _run_thread ($self) {
         my $proc;
 
         while () {
-            $self->{_signal}->wait;
-
           REDO:
             last if !$self;
 
@@ -179,6 +180,8 @@ sub _run_thread ($self) {
 
                 goto REDO;
             }
+
+            $self->{_signal}->wait;
         }
 
         $self->{_threads}--;
@@ -275,9 +278,9 @@ sub _run_task ( $self, $task, $proc ) {
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
 ## |    2 |                      | Documentation::RequirePodLinksIncludeText                                                                      |
-## |      | 285                  | * Link L<Pcore::Util::Result> on line 342 does not specify text                                                |
-## |      | 285                  | * Link L<Pcore::Util::Result> on line 352 does not specify text                                                |
-## |      | 285                  | * Link L<Pcore> on line 350 does not specify text                                                              |
+## |      | 288                  | * Link L<Pcore::Util::Result> on line 345 does not specify text                                                |
+## |      | 288                  | * Link L<Pcore::Util::Result> on line 355 does not specify text                                                |
+## |      | 288                  | * Link L<Pcore> on line 353 does not specify text                                                              |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
